
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class V1::ComplaintsController < ApplicationController
  include ::DeletedObjectsController
  include ::Paginator


  before_action :set_complaint, only: [:show, :reject, :accept]
  before_action :authenticate_user!, only: [:index, :show, :create, :reject, :accept]

  # GET v1/complaints
  # GET v1/complaints.json
  def index
    complaints = paginate Complaint
    render json: complaints
  end

  # GET /v1/coplaints/1
  # GET /v1/coplaints/1.json
  def show
    render :json => @complaint, :include => {:complaint_reason => {:except => [:updated_at, :created_at]}}
  end

  # POST v1/complaints
  # POST v1/complaints.json
  def create
    @complaint = Complaint.new(complaint_creation_params)
        
    if @complaint.save
      ComplaintsMailer.new_complaint_received(@complaint, @current_user).deliver_now

      @complaint.complainable.treat_complaintment

      render json: @complaint, status: :created
    else
      render json: @complaint.errors, status: :unprocessable_entity
    end
  end

  # POST v1/complaints/1/reject
  # POST v1/complaints/1/reject.json
  def reject
    @complaint.rejected!
    @complaint.complainable.complaint_reject(complaint_answer_params)
    response = { 'status': 'complaint rejected' }
    render status: :ok, json: response
  end

  # POST v1/complaints/1/accept
  # POST v1/complaints/1/accept.json
  def accept
    @complaint.accepted!
    @complaint.complainable.complaint_accept(complaint_answer_params)
    response = { 'status': 'complaint accepted' }
    render status: :ok, json: response
  end

  private

  def deleted_resource
    Complaint
  end

  def complaint_creation_params
    params.require(:complaint).permit(:user_id, :description,:complainable_id, :complaint_reason_id, :complainable_type)
  end

  def complaint_answer_params
    params.require(:complaint).permit(:id, :option)    
  end

  def set_complaint
    @complaint ||= Complaint.find(params[:id])
  end
end
